﻿ltc = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceHook-2.1", "AceConsole-2.0", "AceDB-2.0")
local Gratuity = AceLibrary("Gratuity-2.0")
local Compost = AceLibrary("Compost-2.0")

local Plugins = Compost:GetTable ()
local CostList = Compost:GetTable ()
local ReagentList = Compost:GetTable ()
local ReagentCount = Compost:GetTable ()
local CountList = Compost:GetTable ()

local L = AceLibrary("AceLocale-2.2"):new("lern2count")

local CostTable = {
	{ type = 0, base = MANA_COST, timed = MANA_COST_PER_TIME }, 
	{ type = 1, base = RAGE_COST, timed = RAGE_COST_PER_TIME },
	{ type = 2, base = FOCUS_COST, timed = FOCUS_COST_PER_TIME },
	{ type = 3, base = ENERGY_COST, timed = ENERGY_COST_PER_TIME },
	{ type = 4, base = HEALTH_COST, timed = HEALTH_COST_PER_TIME },
}

ltc:RegisterDB("lern2countDB")
ltc:RegisterDefaults('profile', {
	colors = {
		[0] = "88aaff",
		[1] = "ffff33",
		[2] = "ff8033",
		[3] = "ffff33",
		[4] = "ff0033",
		R   = "00ff33",
		D   = "ffffff",
	},
	sizes = {
		[0] = 14,
		[1] = 14,
		[2] = 14,
		[3] = 14,
		[4] = 14,
		R   = 14,
		D   = 14,
	},
	hidden = {
		[0] = false,
		[1] = false,
		[2] = false,
		[3] = false,
		[4] = false,
		R   = false,
		D   = false,
	},
})

local function getRGB(hex)
	local r = tonumber(string.sub(hex,1,2), 16) / 255
	local g = tonumber(string.sub(hex,3,4), 16) / 255
	local b = tonumber(string.sub(hex,5,6), 16) / 255
	return r, g, b
end

local function setRGB(r, g, b)
	local hex = string.format("%02x%02x%02x", math.floor(r * 255), math.floor(g * 255), math.floor(b * 255))
	return hex
end

function ltc:OnInitialize()
	local nameMap = {
		[0] = L["mana"],
		[1] = L["rage"],
		[2] = L["focus"],
		[3] = L["energy"],
		[4] = L["health"],
		R   = L["reagent"],
		D   = L["count"],
	}
	local options = {
		desc = L["An addon to show reagent count on action buttons"],
		type = "group",
		args = {
			colors = {
				name = L["colors"],
				desc = L["Color of the text for the different elements"],
				type = "group",
				args = {},
			},
			sizes = {
				name = L["sizes"],
				desc = L["Size of the text for the different elements"],
				type = "group",
				args = {},
			},
			hidden = {
				name = L["hide"],
				desc = L["Hide the text for the different elements"],
				type = "group",
				args = {},
			},
		}
	}
	for k, v in pairs(nameMap) do
		local index = k
		options.args.colors.args[v] = {
			name = v,
			desc = string.format(L["Color for %s"], v),
			type = "color",
			get = function () return getRGB(self.db.profile.colors[index]) end,
			set = function (r, g, b) self.db.profile.colors[index] = setRGB(r, g, b) ; self:UpdateAllCounts() end,
		}
		options.args.sizes.args[v] = {
			name = v,
			desc = string.format(L["Fontsize for %s"], v),
			type = "range",
			min = 6,
			max = 20,
			step = 1,
			get = function () return self.db.profile.sizes[index] end,
			set = function (v) self.db.profile.sizes[index] = v ; self:UpdateAllCounts() end,
		}
		options.args.hidden.args[v] = {
			name = v,
			desc = string.format(L["Hide %s"], v),
			type = "toggle",
			get = function () return self.db.profile.hidden[index] end,
			set = function (v) self.db.profile.hidden[index] = v ; self:UpdateAllCounts() end,
		}
	end
	
	self:RegisterChatCommand(L["CONSOLE_CMDS"], options)
end

function ltc:OnEnable()
	self:RegisterBucketEvent("UNIT_MANA", 0.2, "PowerUpdate")
	self:RegisterBucketEvent("UNIT_RAGE", 0.2, "PowerUpdate")
	self:RegisterBucketEvent("UNIT_ENERGY", 0.2, "PowerUpdate")
	if not AceLibrary:HasInstance("SpecialEvents-Bags-2.0") then
		self:RegisterEvent("BAG_UPDATE")
	else
		self:RegisterEvent("SpecialEvents_BagSlotUpdate")
	end

	self:RegisterEvent("ACTIONBAR_SLOT_CHANGED","BarUpdate")
	self:RegisterEvent("ACTIONBAR_HIDEGRID","BarUpdate")
	self:RegisterEvent("ACTIONBAR_PAGE_CHANGED","BarUpdate")
	self:RegisterEvent("ACTIONBAR_UPDATE_USABLE","BarUpdate")
	self:RegisterEvent("ACTIONBAR_PAGE_CHANGED","BarUpdate")
	self:RegisterEvent("PET_BAR_HIDEGRID","BarUpdate")
	self:RegisterEvent("PET_BAR_UPDATE","BarUpdate")
	self:RegisterEvent("PET_BAR_UPDATE_COOLDOWN","BarUpdate")

    self:ScheduleEvent(self.DelayedUpdate, 1, self) --Delay initializing the addon a second, thats enough for everything to finish what its doing (I hope)

	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	
	self:Hook("ActionButton_UpdateCount", nil, true)

end

function ltc:PLAYER_REGEN_DISABLED()
	self.combat = true
	self:UpdateAllCounts()
end

function ltc:PLAYER_REGEN_ENABLED()
	self.combat = false
	self:UpdateAllCounts()
end

local GratuityHandler = {
	action = function (id) Gratuity:SetAction (id) end,
	pet = function (id) Gratuity:SetPetAction (id) end,
	shapeshift = function (id) Gratuity:SetShapeshift (id) end,
}

local math_abs = math.abs
function ltc:UpdateCount(frame, info)
	local pdebug = frame:GetName() == "PetActionButton2Count"
	
	local ptype = UnitPowerType("player")

	GratuityHandler[info.type](info.id)
	
	local ctype, cost
	do
		local cl = CostList[frame]
		if cl then
			ctype = cl.type
			cost = cl.cost
		end
	end
	
	local amount
	if info.type == "pet" then
		amount = UnitMana("pet")
	elseif ctype == 4 then
		amount = UnitHealth("player")
	else
		amount = UnitMana("player")
	end
	
	local rcount
	do
		local reagent = ReagentList[frame]
		if reagent then
			rcount = ReagentCount[reagent]
			if not rcount then
				rcount = self:GetInvCount (reagent)
				ReagentCount[reagent] = rcount
			end
		end
	end
	
	local pcount
	if cost then
		if ctype == 3 and ptype == 3 then -- Energy
			pcount = ceil( (cost - amount) / 20 )
			if pcount <= 0 then pcount = nil end
		else
			pcount = floor(amount / cost)
		end
		if pcount == 0 and ctype == 1 then -- Rage
			pcount = nil
		end
	end

	local value
	if self.combat then
		if rcount and (not pcount or rcount < pcount) then
			value =  rcount
			ctype = "R"
		else
			value = pcount
		end
	else
		if rcount then
			value = rcount
			ctype = "R"
		else
			value = pcount
		end
	end
		
	if not value and info.type == "action" and IsConsumableAction (info.id) then
		value = GetActionCount (info.id)
		ctype = "D"
	end
	
	if pdebug then
		self:Print("ctype", ctype, "cost", cost, "amount", amount, "rcount", rcount, "pcount", pcount, "value", value)
	end

	local p = self.db.profile
	if value and ctype and not p.hidden[ctype] then
		frame:SetText("|cff"..p.colors[ctype]..value.."|r")
		local new_height = p.sizes[ctype]
		
		local font, height, flag = frame:GetFont()
		if math_abs(height - new_height) > .5 then
			frame:SetFont(font, new_height, flag)
		end
	else
		frame:SetText()
	end
end

function ltc:GetCountList()
	return CountList
end

function ltc:UpdateAllCounts()
	for frame, info in pairs(CountList) do
		self:UpdateCount (frame, info)
	end
end

function ltc:ActionButton_UpdateCount()
	local frame = getglobal(this:GetName().."Count")
	local info = CountList[frame]
	if info then
		self:UpdateCount (frame, info)
	else
		self.hooks.ActionButton_UpdateCount ()
	end
end

function ltc:DelayedUpdate()

	self:CountListUpdate()
	self:CostAndReagentListUpdate()
	self:UpdateAllCounts()

end

function ltc:RegisterPlugin(func)
	table.insert(Plugins, func)
end

function ltc:CountListUpdate()
	Compost:Reclaim(CountList, 2)
	CountList = Compost:GetTable ()

	for _, func in ipairs(Plugins) do
	 	func (CountList)
	end
end

function ltc:CostAndReagentListUpdate()
	Compost:Reclaim(CostList, 2)
	CostList = Compost:GetTable ()
	
	ReagentList = Compost:Erase (ReagentList)
	ReagentCount = Compost:Erase (ReagentCount)
	
	for frame, info in pairs(CountList) do
		GratuityHandler[info.type](info.id)
		if Gratuity:NumLines() >= 2 then
		
			local pdebug = frame:GetName() == "PetActionButton2Count"
			
			for _,c in pairs(CostTable) do
				local cost = Gratuity:FindDeformat(c.base, 2, 2, false, true) or Gratuity:FindDeformat(c.timed, 2, 2, false, true)
				if pdebug then
					self:Print("type", info.type, "id", info.id, "cost", 10, "lines", Gratuity:NumLines(), "line", Gratuity:GetLine(2), "start", Gratuity:GetLine(1))
				end

				if cost then
					cost = tonumber(cost)
					CostList[frame] = Compost:AcquireHash ('type', c.type, 'cost', cost)
					break
				end
			end
			
			local count
			local _,_,reagent = Gratuity:Find(SPELL_REAGENTS.."(.+)", 2, 99, false, true)
			if reagent then
				if string.sub(reagent, 0, 2) == "|c" then
					reagent = string.sub(reagent, 11, -3)
				end
				ReagentList[frame] = reagent
			end
		end
	end
end

local function GetItemName(link)
	local _, _, id = string.find(link, "item:(%d+):%d+:%d+:%d+")
	if id then
		id = tonumber (id)
		if id then
			local name = GetItemInfo(id)
			return name
		end
	end
end

function ltc:SpecialEvents_BagSlotUpdate(bag, slot, itemlink, stack, oldlink, oldstack)
	if oldlink then
		local reagent = GetItemName(oldlink)
		if reagent then
			local count = ReagentCount[reagent]
			if count then
				ReagentCount[reagent] = count - oldstack
			end
		end
	end
	if itemlink then
		local reagent = GetItemName(itemlink)
		if reagent then
			count = ReagentCount[reagent]
			if count then
				ReagentCount[reagent] = count + stack
			end
		end
	end
end

function ltc:BAG_UPDATE()
	ReagentCount = Compost:Erase (ReagentCount)
end


function ltc:PowerUpdate(units)
	if units.player or units.pet then
		self:UpdateAllCounts()
	end
end

function ltc:GetInvCount(item)
	local val = 0
	for b = 0, NUM_BAG_FRAMES do
     	for s = 1, GetContainerNumSlots(b) do
			local c = GetContainerItemLink(b, s)
			if c then
				local cName = GetItemName (c)
				if cName == item then
					local _, amount = GetContainerItemInfo(b, s)
					val = val + amount
				end
			end
		end
	end
	return val
end

function ltc:BarUpdate(slot)
	self:ScheduleEvent(self.DelayedUpdate, 0.1, self)
end
